
program browse;
{   Written by R. Israel: contributed to public domain.
    For Turbo Pascal under MSDOS/PCDOS.  To use paths you need at least Turbo
    version 3.0 and DOS version 2.0.

    This program demonstrates two things:
       Directory search and file moving using DOS function calls
       Long gotos using inline code
    You may also find it useful in itself.
    The main menu has the following options:
         0: Quit
         1: Type a file
         2: Make new directory
         3: Move/rename a file
         4: Change directory
      In option 1, you enter a file spec, which may include "wild card" 
    characters ? and *. The program searches the given (or default) directory 
    for files matching the specification, and displays them one at a time with 
    time, date, and size.  When you indicate the one you want by pressing "Y", 
    the file is typed.
      Option 2 uses the MkDir procedure to make a new directory (nothing fancy 
    here, but it fits well with the other options).
      Option 3 renames a file, or moves it to a new directory (on the same 
    drive).  Again the file name may contain "wild cards", and matches are 
    displayed one by one as in option 1.  The new path (including file name) 
    must be given exactly, without "wild cards".
      Option 4 uses the ChDir procedure to change the default directory (and 
    drive if desired).  The new directory name at the end of the path may 
    contain "wild cards", in which case a search is made for matches.  (Wild 
    cards are not allowed before the end, e.g. \???\pas is not allowed).
}

type textline = string[80];
     str13 = string[13];
     arrchar = array[1..13] of char;
     lint = record              { double length integer, 0 to 2^32-1 }
            loword,hiword:integer;
            end;
     direntry = record
               res:array[1..21] of byte;
               atr: byte;
               tim,dat: integer;
               siz: lint;
               pnam: arrchar;
               end;
     regpack = record
               AX,BX,CX,DX,BP,SI,DI,DS,ES,Flags:integer;
               end;

(*  LONG GOTOS
       Turbo Pascal allows gotos only within a single block, so it is not 
     possible to jump into or out of procedures or functions.  It is often 
     useful, on the other hand, to have an "emergency exit" available when, 
     for example, an error condition is discovered in a procedure.  Turbo has 
     EXIT, but this only returns one level, and the caller would need some way 
     of distinguishing between a normal and abnormal return.  The procedures 
     SETJUMP and ABORT allow a return to a preset point through any number of 
     layers of calls.  
       SETJUMP sets the return address just after the point where it is 
     called.  It must be called before ABORT.  ABORT will then jump to the 
     return address set by SETJUMP.  For example, in this program each ABORT 
     will cause a return to the main menu.  
       You should never allow an ABORT to return to a procedure or function 
     that has terminated, as its frame will no longer be on the stack.  For 
     example, the following would be WRONG: 
         program xxxx;
         procedure SETJUMP; ...
         procedure ABORT; ...
         procedure proc1;
           begin ...; SETJUMP; ... end;
         procedure proc2;
           begin ...; ABORT; ... end;
         begin { main }
          proc1; proc2;
         end.
     This is wrong because the ABORT in proc2 will cause a jump into proc1,
     which has returned.  It would be legal if the call to SETJUMP were in
     the main program or if the call to proc2 were in proc1.
*)

var bpsave,rasave:integer;
{ global variables saving the frame pointer and return address }
    infname,newname:textline; c:char; quit:boolean;

procedure SETJUMP;
{ set up for ABORT to return to point just after the call to SETJUMP }
begin
inline($89/$2e/bpsave/$8b/$46/$02/$a3/rasave);
{ assembly code:
  mov BPSAVE,BP  ; this is the frame pointer for SETJUMP
  mov AX,[BP+2]  ; this is the return address
  mov RASAVE,AX }
end;

procedure ABORT;
{ returns to last point where SETJUMP was called }
begin
inline($8b/$2e/bpsave/$a1/rasave/$89/$46/$02);
{ assembly code:
    mov BP,BPSAVE  ; reset frame pointer
    mov AX,RASAVE  ; get return address
    mov [BP+2],AX  ; put on stack
  The normal procedure-ending code generated by Turbo is then
    mov SP,BP
    pop BP
    ret
  so that the "ret" will return to the return address from RASAVE }
end;

function GETZ(var st:arrchar):str13;
{    st is an ASCIIZ string, i.e. it has no length byte but is terminated
     by char 0.  This function converts it to a Turbo string    }
var sp:str13;
begin
sp:= st;
delete(sp,pos(#0,sp),13);
getz:= sp;
end;

function GETPATH(var st:textline):textline;
{ extract path from incomplete filespec }
var m,n:integer;
begin
{ first try last '\' }
n:= 0;
for m:= 1 to ord(st[0]) do
  if st[m] = '\' then n:= m;
if n <> 0 then getpath:= copy(st,1,n)
  else
    { no path; is there a drive? }
    if st[2] = ':' then getpath:= copy(st,1,2) else getpath:= '';
end;

procedure TYPEFILE(infname: textline);
{ type a file }
var infile:text; tline:textline;

  procedure OPENINFILE(infname: textline);
{ open input file }
  begin
  assign(infile,infname);
{$I-} reset(infile); {$I+}
  if ioresult <> 0 then
    begin
    writeln('Can''t open ',infname);
    ABORT
    end
  end;  { Openinfile }

begin { Typefile }
OPENINFILE(infname);
while not eof(infile) do
  begin
  readln(infile,tline);
  writeln(tline);
  end;
writeln('<<< End of file >>>');
close(infile);
end;   { Typefile }

function MAKEDOUBLE(x:lint):real;
{ convert a double-length integer to real }
var y:real;
begin
y:= x.loword;
if y < 0 then y:= 65536.0+y;
if x.hiword < 0 then makedouble:= y+65536.0*(65536.0+x.hiword)
  else makedouble:= y+65536.0*x.hiword;
end;    { makedouble }

function TIMESTR(x:integer): str13;
{ convert integer x to time string:
  hours in top 5 bits, minutes next 6, 2*seconds low 5 }
var hourstr,minstr,secstr: str13; i:integer;
begin
str((x and $f800) shr 11:2,hourstr);   { hours }
str((x and $7e0) shr 5:2,minstr);      { minutes }
str((x and $1f) shl 1:2,secstr);       { seconds }
hourstr:= hourstr+':'+minstr+':'+secstr;
for i:= 1 to 8 do   { convert blanks to zeros }
  if hourstr[i] = ' ' then hourstr[i]:= '0';
timestr:= hourstr;
end;        { timestr }

function DATESTR(x:integer):str13;
{  convert integer x to date string:
   year - 1980 is top 15 bits, month next 4, day low 5 }

var yearstr,monstr,daystr:str13;
begin
str((x and $fe00) shr 9 + 1980,yearstr);
str((x and $1e0) shr 5:2,monstr);
str(x and $1f:2,daystr);
if daystr[1] = ' ' then daystr[1]:= '0';
datestr:= monstr+'-'+daystr+'-'+yearstr;
end;     { datestr }


procedure SEARCH(var infname:textline; attrib:byte);
{ on input, infname is incomplete file spec including wild cards.
  attrib sets attributes to include (e.g. $10 includes directories).
  on normal exit, complete file spec for selected file
  exits via ABORT for invalid path or no (more) matches }

var i:integer; de:direntry; regs:regpack; fn:str13; c:char;
begin
with regs do
    begin
    infname:= infname+#0;  { DOS works with ASCIIZ strings }
    DS:= seg(de); DX:= ofs(de);  { buffer for file info }
    AX:= $1A00;            { function 1A: set DTA }
    msdos(regs);
{ note: the DTA must be reset before using functions 4E or 4F if any disk
  I/O has occurred since the last time it was set }
    AX:= $4E00;          { function 4E: find first match to filespec }
    DS:= seg(infname);
    DX:= ofs(infname[1]);  { skip length byte }
    CX:= attrib;   {  attributes }
    c:= 'n';
    repeat
      msdos(regs);
      if (flags and 1) <> 0 then   { carry flag set for error }
          begin
          case AX of
            2: writeln('Invalid path');
            18: writeln('No matches');
            end;    { cases }
          ABORT
          end
        else with de do
          begin
          AX:= $4F00;        { function 4F: next match }
          fn:=getz(pnam);
          write(fn:13,datestr(dat):13,timestr(tim):13);
          if (atr and $10) <> 0 then writeln(' <DIR>')
            else writeln(makedouble(siz):13:0,' bytes');
          write('        OK? (Y/N) '); read(kbd,c); writeln;
          end;
      until upcase(c) = 'Y';
    infname:= getpath(infname)+fn;
    end;  { with }
end;  { search }

procedure movedir(infname,newname:textline);
{ rename infname to newname (which may involve changing its
  directory) }

var regs:regpack;
begin
infname:= infname+#0;            { make ASCIIZ strings }
newname:= newname+#0;
with regs do
  begin
  AX:= $5600;                    { function $56: rename }
  DX:= ofs(infname[1]); DI:= ofs(newname[1]); { skip length bytes }
  DS:= Sseg; ES:= Sseg;          { both strings are on stack }
  MsDos(regs);
  if (flags and 1) <> 0 then     { carry flag set if error }
    case AX of
      2: writeln('File not found');
      17: writeln('Not same device');
      5: writeln('Access denied');
      end   { cases }
   else writeln('Done');
  end;
end;

function mainmenu:integer;
var c:char; m,code:integer;
begin
writeln('Enter your choice:');
writeln('0: Quit');
writeln('1: Type a file');
writeln('2: Make new directory');
writeln('3: Move/rename a file');
writeln('4: Change directory');
readln(c);
val(c,m,code);
if code <> 0 then ABORT;
mainmenu:= m;
end;

begin {main program }
quit:= false;
SETJUMP;         { set return address for ABORT }
repeat
  case mainmenu of
    0: quit:= true;
    1: begin     { type a file }
       write('Enter file spec '); readln(infname);
       SEARCH(infname,0);
       TYPEFILE(infname);
       end;
    2: begin     { set up new directory }
       write('New directory? '); readln(newname);
       {$I-} mkdir(newname); {$I+}
       if ioresult = 0 then writeln('Done')
        else writeln('Can''t be done');
       end;
    3: begin     { move a file }
       write('File to transfer? '); readln(infname);
       SEARCH(infname,0);
       write('New path and file name? '); readln(newname);
       movedir(infname,newname);
       end;
    4: begin     { change directory }
       write('New directory? '); readln(newname);
       if pos('*',newname) or pos('?',newname) <> 0 then
         SEARCH(newname,$10);
       {$I-} chdir(newname); {$I+}
       if ioresult = 0 then writeln('Done')
        else writeln('Can''t be done');
       end;
    else ABORT;
    end;
  until quit;
end.









