
{$C-}

{
reads a master DSDD disk into memory.
writes it to drives A and B in alternation without rereading the master.
requires 2 floppy drives (could be modified for a single one).
works for DSDD DOS 2.0+ 360K floppies only.
works for PC-DOS only.

written 7/28/86, Kim Kokkonen, TurboPower Software
Compuserve 72457,2131.

released to the public domain.
}

program fcopy;
  {-fast DSDD disk copier}

const
  drivea = 0;
  driveb = 1;

type
  {holds sector data during sector setup after formatting}
  SectorBuffer = array[1..512] of Char;
  TrackBuffer = array[1..9] of SectorBuffer;
  TrackPtr = ^TrackBuffer;
  diskarray = array[0..39, 0..1] of TrackPtr;
  registers = record
                case Integer of
                  1 : (ax, bx, cx, dx, bp, si, di, ds, es, flags : Integer);
                  2 : (al, ah, bl, bh, cl, ch, dl, dh : Byte);
              end;

var
  reg : registers;
  ch : Char;
  data : diskarray;
  count : Integer;
  t1, t2 : Real;

  procedure BIOSreadSectors(funct, drive : Byte;
                            sector, track, head : Integer;
                            sects : Integer;
                            var buffer;
                            var error : Integer);
    {-execute int 13 to read disk or verify via BIOS at low level}
  begin
    reg.ax := (funct shl 8) or sects;
    reg.dl := drive;
    reg.dh := head;
    reg.ch := track and 255;
    reg.cl := (sector and 63) or ((track shr 8) shl 6);
    reg.es := Seg(buffer);
    reg.bx := Ofs(buffer);
    Intr($13, reg);
    if Odd(reg.flags) then
      error := reg.ax shr 8
    else
      error := 0;
  end {BIOSreadsectors} ;

  procedure BIOSwriteSectors(drive : Byte;
                             sector, track, head : Integer;
                             sects : Integer;
                             var buffer : TrackBuffer;
                             var error : Integer);
    {-execute int 13 to write disk via BIOS at low level}
  begin
    reg.ax := $300 or sects;
    reg.dl := drive;
    reg.dh := head;
    reg.ch := track and 255;
    reg.cl := (sector and 63) or ((track shr 8) shl 6);
    reg.es := Seg(buffer);
    reg.bx := Ofs(buffer);
    Intr($13, reg);
    if Odd(reg.flags) then
      error := reg.ax shr 8
    else
      error := 0;
  end {BIOSwritesectors} ;

  procedure BIOSdiskReset;
    {-reset the diskettes after an error}
  begin
    reg.ah := 0;
    Intr($13, reg);
  end {BIOSdiskReset} ;

  function Cardinal(i : Integer) : Real;
    {-return a real 0..65535}
  var
    r : Real;
  begin                       {Cardinal}
    r := i;
    if r < 0.0 then r := r+65535.0;
    Cardinal := r;
  end {Cardinal} ;

  procedure ReadMasterDisk(drive : Byte);
    {-read a full DSDD floppy from drive A}
  var
    t, s, h : Integer;
    error : Integer;
    ch : Char;
    tries : Integer;
  begin                       {ReadMasterDisk}
    Write(0:2);
    for t := 0 to 39 do begin
      Write(^H^H, t:2);
      for h := 0 to 1 do begin
        New(data[t, h]);
        tries := 0;
        repeat
          tries := Succ(tries);
          BIOSreadSectors(2, drive, 1, t, h, 9, data[t, h]^, error);
          if error <> 0 then
            BIOSdiskReset;
        until (error = 0) or (tries > 3);
        if error <> 0 then begin
          Write(^H^H'  ');
          WriteLn;
          WriteLn('Cannot read master disk'^G);
          Write('Insert diskette and press a key... ');
          Read(Kbd, ch);
          WriteLn;
        end;
      end;
    end;
    Write(^H^H'  ');
  end {ReadMasterDisk} ;

  procedure WriteDisk(drive : Byte);
    {-read a full DSDD floppy from drive A}
  label
    retry;
  var
    t, s, h : Integer;
    error : Integer;
    good : Boolean;
    ch : Char;

    function time : Real;
      {-return time of day in seconds since midnight}
    begin
      reg.ah := $2C;
      MsDos(reg);
      time := 1.0*(reg.dh+60.0*(reg.cl+60.0*reg.ch)+reg.dl/100.0);
    end {time} ;

    procedure breakpressed;
      {-halt if break is pressed}
    var
      ch : Char;
    begin                     {breakpressed}
      while KeyPressed do begin
        Read(Kbd, ch);
        if ch = ^C then begin
          WriteLn('^C');
          Halt(2);
        end;
      end;
    end {breakpressed} ;

  begin

    repeat

      breakpressed;
      WriteLn;
      Write(count:3, ' Writing drive ', Chr(drive+65), ' ');
      t1 := time;
      Write(0:2);

      for t := 0 to 39 do begin
        Write(^H^H, t:2);
        for h := 0 to 1 do begin
          BIOSwriteSectors(drive, 1, t, h, 9, data[t, h]^, error);
          if error <> 0 then begin
            t2 := time;
            WriteLn(^H^H, t2-t1:0:2, ' secs'^G);
            WriteLn;
            if (error and $80) <> 0 then
              {timeout}
              Write('  Drive not ready. Insert disk and press a key... ')
            else if (error and 3) <> 0 then
              {write protect}
              Write('  Disk is write-protected. Insert another disk and press a key... ')
            else
              {bad disk}
              Write('  Disk may be unusable. Press a key or try another disk... ');
            repeat until KeyPressed;
            WriteLn;
            good := False;
            BIOSdiskReset;
            goto retry;
          end;
        end;
      end;

      count := Succ(count);
      good := True;
      t2 := time;
      WriteLn(^H^H, t2-t1:0:2, ' secs');

retry: 

    until good;

  end {WriteDisk} ;


begin

  WriteLn;

  {check for sufficient memory}
  if 16.0*Cardinal(MemAvail) < 370000.0 then begin
    WriteLn('Insufficient RAM space to read master disk');
    Halt;
  end;

  {prompt to read from drive a:}
  Write('Insert master disk into drive A. Press a key when ready... ');
  Read(Kbd, ch);
  WriteLn;

  {read the disk into memory}
  ReadMasterDisk(drivea);

  WriteLn;
  WriteLn('Insert first disk to copy in drive A.');
  WriteLn('Copies will alternate from A to B. Press ^C to stop.');
  Write('Press a key when ready... ');
  Read(Kbd, ch);
  WriteLn;
  WriteLn;
  count := 1;

  {loop copying to drives a and b}
  repeat
    WriteDisk(drivea);
    WriteDisk(driveb);
  until False;

end.
:= 1;

  {loop copying to drives a and b}
  repeat
    WriteDisk(drivea);
    WriteDisk(driveb);
  until False;

end.
